#-*- Mode: Perl -*-

## File: Pd/Receive.pm
## Author: Bryan Jurish <moocow@ling.uni-potsdam.de>
## Description:
##  + Pd netsend interface for perl
##======================================================================

package Pd::Socket::Receive;
use Socket;
use IO::Socket;
use IO::Socket::INET;

@ISA = qw(IO::Socket::INET);

##======================================================================
## Generic Constructor
##======================================================================

#our $DEBUG = 0;

our %DEFAULTS =
  (LocalAddr=>'127.0.0.1',
   Proto=>'tcp',
   Type=>SOCK_STREAM,
   #Listen=>SOMAXCONN,
   ReuseAddr=>1,
   #ReusePort=>1,
   Timeout=>0);

## $sock = Pd::Socket::Receive->new(%args);
##   + %args keys: LocalAddr, LocalPort, etc.
##     - see IO::Socket::INET
sub new {
  my ($that,%args) = @_;
  my ($k,$v);
  while (($k,$v) = each(%DEFAULTS)) {
    $args{$k} = $v if (!exists($args{$k}));
  }
  #if ($DEBUG) {
  #  ##--debug
  #  print "", __PACKAGE__ , "::new($that, ";
  #  foreach $k (sort keys(%args)) {
  #    print "$k='$args{$k}' ";
  #  }
  #  print ")\n";
  #}
  my $self = IO::Socket::INET->new(%args);
  return defined($self) ? bless($self, ref($that) || $that) : undef;
}

##======================================================================
## Pd::Socket::Receive::TCP
##======================================================================

package Pd::Socket::Receive::TCP;
use Socket;
use IO::Socket;
use IO::Socket::INET;
@ISA = qw(Pd::Socket::Receive);

## $sock = Pd::Socket::Send::TCP->new(%args);
##   + %args keys:
##     - see IO::Socket::INET
sub new {
  my $that = shift;
  return Pd::Socket::Receive::new($that,
				  Listen=>SOMAXCONN,
				  @_,
				  Proto=>'tcp',
				  Type=>SOCK_STREAM);
}

##======================================================================
## Pd::Socket::Receive::UDP
##======================================================================

package Pd::Socket::Receive::UDP;
use Socket;
use IO::Socket;
use IO::Socket::INET;
@ISA = qw(Pd::Socket::Receive);

## $sock = Pd::Socket::Receive::UDP->new(%args);
##   + %args keys:
##     - see IO::Socket::INET
sub new {
  my $that = shift;
  return Pd::Socket::Receive::new($that,
				  @_,
				  Proto=>'udp',
				  Type=>SOCK_DGRAM);
}



##======================================================================
## Docs
=pod

=head1 NAME

Pd::Socket::Receive - Perl interface to Pd [netreceive].

=head1 SYNOPSIS

 use Pd::Socket::Receive;

 # Create a socket...
 $tcp_sock = Pd::Socket::Receive::TCP->new(LocalPort=>3000,Listen=>1);
 $udp_sock = Pd::Socket::Receive::UDP->new(LocalPort=>3001,Listen=>1); # broken!

 # ... and wait for some data

 # (try the above with the Pd [netreceive] help patch open)

=cut

##======================================================================
## Description
=pod

=head1 DESCRIPTION

Pd::Socket::Receive provides a Perl API similar to the
PureData [netreceive] object.

=cut

##======================================================================
## Footer
=pod

=head1 ACKNOWLEDGEMENTS

perl by Larry Wall.

Pd by Miller Puckette and others.

=head1 AUTHOR

Bryan Jurish E<lt>jurish@ling.uni-potsdam.deE<gt>

=head1 COPYRIGHT

Copyright (c) 2004, Bryan Jurish.  All rights reserved.

This package is free software.  You may redistribute it
and/or modify it under the same terms as Perl itself.

=head1 SEE ALSO

perl(1),
pd(1),
pdsend(1),
pdreceive(1),
Pd::Socket(3pm),
Pd::Socket::Receive(3pm).

=cut
